/*
  otak project.
  copyright (C) 2002 
  grzegorz moskal,  <g.moskal@opengroup.org>
  license : gnu gpl version 2 			*/

#define BASECONFIG_C_
#include "baseconfig.h"

struct stanza *first_stanza, *last_stanza, *current_stanza;

static char *label, *command, *key;
static int multiple, text;

void scroll_stanzas(long n)
{
	for (current_stanza = first_stanza; n; n--)
		current_stanza = current_stanza->next;
}

static char *set_string(char *src, char **dest)
{
	if (src)
		*dest = strdup(src);
	else
		*dest = NULL;
		
	return *dest;
}

static void set_varibles(struct ecl_stanza *stanza)
{
	ecl_stanza_string(stanza, "label", &label, NULL);
	ecl_stanza_bool(stanza, "multiple", &multiple, 0);
	ecl_stanza_string(stanza, "command", &command, NULL);
	ecl_stanza_bool(stanza, "text", &text, 0);
	ecl_stanza_string(stanza, "key", &key, NULL);
}

static int add_stanza(char *name)
{
	struct stanza *s = xmalloc(sizeof(struct stanza));
			
	if ((set_string(name, &s->name)) == NULL ||
		set_string(label, &s->label) == NULL)
		return 0;
		
	set_string(command, &s->command);
	s->key = (key) ? *key: 0;
	s->multiple = multiple;		
	s->text = text;
	
	if (first_stanza == NULL)
		current_stanza = first_stanza = last_stanza = s;
	else {
		last_stanza->next = s;
		last_stanza = s;
	}
	
	s->next = NULL;

	return 1;
}


int read_baseconfig(char *name) 
{
	struct ecl_file *file;	
	struct ecl_stanza *s;

	if (name == NULL) 
		return 0;
	
	if ((file = ecl_file_open(name)) == NULL)
			return 0;
	ecl_file_init(file);
	
	msgf("read in base config file:  '%s'", name);

	for (s = file->head; s; SWITCH(s)) {
		set_varibles(s);
		if (add_stanza(s->name) == 0)
			return 0;
	}

	ecl_file_close(file);	
	
	return 1;
}

struct stanza *find_stanza_by_name(char *name)
{
	struct stanza *s = first_stanza;
	for (; s; s = s->next)
		if (!strcmp(s->name, name))
			break;
	return s;
}

void clear_allenv()
{
	struct stanza *s = first_stanza;
	for (; s; s = s->next)
		setenv(s->name, "", 1);
}

void free_stanzas(void)
{
	struct stanza *sj;
	for (current_stanza = first_stanza; current_stanza;) {
		sj = current_stanza;
		current_stanza = sj->next;
		xfree(sj->label);
		xfree(sj->name);
		if (sj->command)
			xfree(sj->command);
		xfree(sj);
	}
}
