/*
  otak project.
  copyright (C) 2002 
  grzegorz moskal,  <g.moskal@opengroup.org>
  license : gnu gpl version 2 			*/

#define BASE_C_
#define BASE_FRIEND_
#include "base.h"

static struct man *mj;
static struct drawer *dj;
static struct item *ij;

static void free_man(struct man *p)
{
	struct drawer *dj2;
	struct item *ij2;

	if (p->last)
		p->last->next = p->unknown;

	for (dj = dj2 = p->first; dj; ) {
		for (ij = ij2 = dj->first; ij; ) {
			ij2 = ij->next;
			xfree(ij);
			ij = ij2;
		}

		dj2 = dj->next;
		xfree(dj);
		dj = dj2;
	}

	xfree(p);
}

void del_man_from_list(struct man *p)
{
	if (!p) {
		first_man = last_man = current_man = 0;
		return;
	}
	for (mj = first_man; mj; mj = mj->next)
		if (mj->next == p) {
			mj->next = p->next;
			break;
		}

	if (p == first_man)
		first_man = p->next;
	if (p == last_man) 
		last_man = mj;
		
}

void del_man(struct man *p)
{	
	del_man_from_list(p);
	if (p)
		free_man(p);
}

void del_item(struct drawer *d, struct item *i)
{
	for (ij = d->first; ij; ij = ij->next)
		if (ij->next == i)
			break;
	if (ij)			
		ij->next = i->next;

	if (i == d->first)
		d->first = i->next;
	if (i == d->last)
		d->last = ij;
	xfree(i);

	// staing with empty drawer. 
}

void del_drawer(struct man *m, struct drawer *d)
{
	if (m->first != d) {
		for (dj = m->first; dj; dj = dj->next)
			if (dj->next == d) {
				dj->next = d->next;
				break;
			}
		if (d == m->last)
			m->last = dj;
	} else {
		if (d->next)
			m->first = d->next;
		else
			m->first = m->last = 0;
	}
	
	xfree(d);
}


static int bigger(struct man *m1, struct man *m2, char *name)
{
	char *str_m1, *str_m2;
	int ret;

	dj = find_drawer_by_name(m1, name);
	str_m1 = (dj) ? strtolower(dj->first->caption) : strdup("");
	
	dj = find_drawer_by_name(m2, name);
	str_m2 = (dj) ? strtolower(dj->first->caption) : strdup("");

	ret = (strcmp(str_m1, str_m2) > 0) ? 1 : 0;
	if (str_m1)
		xfree(str_m1);
	if (str_m2)
		xfree(str_m2);

	return ret;
}

static void change_men(struct man *m1, struct man *m2)
{
	if (m1 != first_man) {
		for (mj = first_man; mj; mj = mj->next)
			if (mj->next == m1)
				break;
		m1->next = m2->next;
		mj->next = m2;
		m2->next = m1;
	} else {
		m1->next = m2->next;
		m2->next = m1;
		first_man = m2;
	}
}

static void write_drawer(FILE *fd, struct drawer *d)
{
	fprintf(fd, "%s=", d->pattern->name);
	for (ij = d->first; ij; ij = ij->next) 
		fprintf(fd, "%s%s", ij->caption, (ij->next) ? "," : "\n");
		
}

static void sort_base(char *name)
{
	struct man *next;
	
	if (!sort_database)
		return;

	for (mj = first_man; mj;) {
		next = mj->next;
		if (next && bigger(mj, next, name)) {
			change_men(mj, next);
			mj = first_man;
		} else
		 	mj = mj->next;
	}
}

void write_base(char *name)
{
	FILE *fd;
	struct stanza *sj;
	int i;

	sort_base(menu_title);

	if (!write_database)
		return;
	
	if (!(fd = fopen(name, "w+"))) {
		errmsgf("Cannot write base to file - '%s'", name);
		return;
	}  
	for (i = 0, mj = first_man; mj; SWITCH(mj), i++) {
		current_man = mj;			// ! find_drawer_by_name use global 
							// pointer current_man [that`s stuped]
		fprintf(fd,"[%d]\n", i);
		for (sj = first_stanza; sj; SWITCH(sj)) 
			if ((dj = find_drawer_by_stanza(sj)))
				write_drawer(fd, dj);
				
		if (mj->unknown) {
			fprintf(fd,"#uknown varibles (supported since 1.3.1 version)\n");
			
			for (ij = mj->unknown->first; ij; SWITCH(ij)) 
				fprintf(fd, "%s\n", ij->caption); 
				// here caption looks like "name=value"
		}

		fprintf(fd,"\n");
	}
	msgf("wrote down base to file:   '%s'", name);
	fclose(fd);
}

void free_men(void)
{	
	struct man *mj2;
	for (mj = first_man; mj;) {
		mj2 = mj;
		mj = mj->next;
		del_man(mj2);
	}
}

