/*
  otak project.
  copyright (C) 2002 
  grzegorz moskal,  <g.moskal@opengroup.org>
  license : gnu gpl version 2 			*/

#define CMDLINE_C_
#include "cmdline.h"

static char 
*usage =
"  %6.0l[ %6.0ahelp %6.0l]\n\n"
" %6.0ausage: otak [%6.0loption ...%6.0a]\n\n"
" %6.0l-h %6.0aor --help                    print this message.\n"
" %6.0l-V %6.0aor --version                 output version info and exit.\n"
" %6.0l-c %6.0aor --config      <filename>  read config from this file.\n"
" %6.0l-B %6.0aor --baseconfig  <filename>  read baseconfig from this file.\n"
" %6.0l-b %6.0aor --basefile    <filename>  read database from this file.\n"
" %6.0l-v %6.0aor --verbose                 work noisily (diagnostic output)\n"
" %6.0a   %6.0a   --name        <string>    use given string as a program name.\n"
" %6.0l-s %6.0aor --no-messages             suppress error messgess.\n"
" %6.0l-Q %6.0aor --quiet\n"
"       --silent                  do not write anything to\n"
"                                 standard output.\n"
" %6.0l-q %6.0aor --query       <string>    search database for string.\n"
" %6.0l-qa %6.0aor --query-all  <string>    search each database for string.\n"
"                                 type --query a - to display whole query.\n"
" %6.0l-a %6.0aor --add-email\n"
" %6.0l-n %6.0aor --new-email               read an email message from stdin.\n"
"                                 and add sender to the database.\n"
" %6.0l-A %6.0aor --add-email-quiet\n"
" %6.0l-N %6.0aor --new-email-quiet         same as --add-email but do not\n"
"                                 confirm adding.\n",
*version =
" %2.0l [ %2.0acopyright (c) grzegorz moskal%2.0l,%2.0a "
       " g%2.0l.%2.0amoskal%2.0l@%2.0aopengroup.org %2.0l]\n\n" 
"  %2.0athis program comes with %2.0labsolutely no warranty%2.0a;"
       " for details\n"
"  please see the file 'COPYING' supplied with the source code.\n"
"  this is free software, and you are %2.0lwelcome%2.0a to %2.0l"
       "redistribute%2.0a it\n"
"  under certain conditions; again, see 'COPYING' for details.\n"
"  this program is %2.0lreleased %2.0aunder the gnu %2.0lg%2.0aeneral "
	"%2.0lp%2.0aublic %2.0ll%2.0aicense v%2.0l2%2.0a.\n";


static int set_this_arg(char *pattern, char **dest, int check)
{
	if (pattern == NULL || *pattern == '\0') 
		return 0;
		
	if (check && *pattern != '/') 
		*dest = strdglue(xgetcwd(), pattern);
	else
		*dest = strdup(pattern);
			
	return 1;
}

static void set_stdin_arg(char **dest)
{
	do 
		*dest = rline(stdin);
	while (*dest && !strstr(*dest, "From:"));
}

#define CALL_SET_ARG(a, check, optional) \
	do {\
		if (set_this_arg(argv[i+1], a, check)) \
			i++;\
		else if (!optional) {\
			cmsg(usage);\
			return 0;\
		}\
	} while (0)

#define SET_FILE_ARG(a) CALL_SET_ARG(a, 1, 0)
#define SET_ARG(a) CALL_SET_ARG(a, 0, 0)
#define SET_OPTIONAL_ARG(a) CALL_SET_ARG(a, 0, 1)

int parse_cmdline(int argc, char **argv)
{
	int i = 1;
	
	for (; i < argc; i++) 
		switch(optionid(argv[i])) {
		case Help:
			cmsg(usage);
			return 0;
		case Version:
			cmsg("  %2.0a" PACKAGE " v " VERSION "\n");
			cmsg(version);
			return 0;
		case Quiet:
			silent_attr = MESSAGES_AND_ERRORS;
			break;
		case No_messages:
			silent_attr = MESSAGES;
			break;
		case Noise:
			silent_attr = 0;
			break;
		case Name:
			SET_ARG(&program_name);
			break;
		case Config:
			SET_FILE_ARG(&config_name);
			break;
		case Baseconfig:
			SET_FILE_ARG(&baseconfig_name);
			break;
		case Basefile:
			SET_FILE_ARG(&base_name);
			break;
		case Add_email:
			confirm_adding = 1;
		case Add_email_quiet:
			set_stdin_arg(&new_email_arg);
			silent_attr = MESSAGES_AND_ERRORS;
			break;
		case Query_each:
			query_each_database = 1;
		case Query:
			SET_OPTIONAL_ARG(&query_arg);
			if (!query_arg)
				query_arg = "a";
			silent_attr = MESSAGES_AND_ERRORS;
			break;
		default:
			silent = 0;
			msgf("option ``%s'' not recognized :(", argv[i]);
			msgf("try %s --help", argv[0]);
			return 0;
		}
		
	return 1;
}

static int opt(char *pattern, char *src1, char *src2)
{
	if (strcmp(pattern, src2) == 0 || strcmp(pattern, src1) == 0) 
		return 1;

	return 0;
}

#define OPTION(a,b) opt(arg, a, b)

static int optionid(char *arg)
{
	if (OPTION("--help", "-h")) 
		return Help;
	else if (OPTION("--version", "-V")) 
		return Version;
	else if (OPTION("--silent", "-Q"))
		return Quiet;
	else if (OPTION("--quiet", "-Q"))
		return Quiet;
	else if (OPTION("--no-messages", "-s"))
		return No_messages;
	else if (OPTION("-v", "--verbose"))
		return Noise;
	else if (OPTION("--name", ""))
		return Name;
	else if (OPTION("--config", "-c")) 
		return Config;
	else if (OPTION("--baseconfig", "-B")) 
		return Baseconfig;
	else if (OPTION("--basefile", "-b")) 
		return Basefile;
	else if (OPTION("--new-email-quiet", "-N"))
		return Add_email_quiet;
	else if (OPTION("--add-email-quiet", "-A"))  
		return Add_email_quiet;
	else if (OPTION("--new-email", "-n"))
		return Add_email;
	else if (OPTION("--add-email", "-a"))  
		return Add_email;
	else if (OPTION("--query", "-q")) 
		return Query;
	else if (OPTION("--query-all", "-qa"))
		return Query_each;
		
	return 0;
}
