#define ECL_C
#include "ecl.h"

// open new config file
struct ecl_file* ecl_file_open(char *name)
{
	struct ecl_file *file = NULL;
	FILE *fd = fopen(name, "r");

//	if (fd == NULL) {
//		fprintf(stderr, ">> can`t open file: '%s'\n", name);
//		perror(">> ecl_file_open() faild ::");
//	} else {
	if (fd) {
		file = XALLOC(ecl_file);
		file->name = strdup(name);
		file->src = fd;
	}
	
	return file;
}

static void ecl_stanza_free(struct ecl_stanza *s)
{
	struct ecl_caption *to_free;
	
	while (s->head) {
		to_free = s->head;
		SWITCH(s->head);
		xfree(to_free->name);
		xfree(to_free->value);
		xfree(to_free);
	}

	xfree(s->name);
	xfree(s);
}

static void ecl_file_free(struct ecl_file *file)
{
	struct ecl_stanza *to_free;
	
	for (to_free = file->head; file->head;) {
		to_free = file->head;
		SWITCH(file->head);
		ecl_stanza_free(to_free);
	}
	xfree(file->name);
	xfree(file->line);
	xfree(file);
}

// close opened config
void ecl_file_close(struct ecl_file *file)
{
	if (file == NULL)
		return;
		
	fclose(file->src);
	ecl_file_free(file);
}

// read config line from given file
static int ecl_file_get_line(struct ecl_file *file)
{
	xfree(file->line);
	
	if ((file->line = rline(file->src)))
		file->number++;

	return file->number;
}

static void ecl_error(struct ecl_file *file, char *error_text)
{
	fprintf(stderr, ">> %d: Syntax error - %s\n", file->number, error_text);
	fprintf(stderr, ">> %d: %s\n", file->number, file->line);
}

// parse config line f.g '    foo   =  bar'
static void ecl_file_parse_line(struct ecl_file * file, struct ecl_stanza *stanza)
{
	struct ecl_caption *caption;
	char *line = file->line;
	char *name = NULL, *value = NULL, *tmp;

	// skiping white spaces in '   foo'
	for (; *line && isspace(*line); line++);
	if (*line == '\0') {
		ecl_error(file, "only white spaces in line");
		return;
	}
	// skiping whire spaces in 'foo   ='
	for (tmp = line; *tmp; tmp++) 
		if (*tmp == '=') {
			char *tmp2 = tmp;
			for (; tmp2-1 != line && isspace(*(tmp2-1)); tmp2--);
			*(tmp2) = '\0';
			name = strdup(line);
			tmp++;
			break;
		}
		
	if (name == NULL) {
		ecl_error(file, "no '='");
		return;
	}
	// skiping white spaces in '=   bar'
	for (; *tmp && isspace(*tmp); tmp++);
		
	if (tmp) {
		value = strdup(tmp);
		
	} else {
		ecl_error(file, "no value given for variable");
		return;
	}

	caption = XALLOC(ecl_caption);
	caption->name = name;
	caption->value = value;
	
	if (stanza->head == NULL)
		stanza->head = caption;
	else {
		struct ecl_caption *tmp = stanza->head;
		for (; tmp->next; SWITCH(tmp));
		tmp->next = caption;
	}
}

// read whole stanza from given file and init ecl_stanza structure
static struct ecl_stanza* ecl_file_build_stanza(struct ecl_file *file)
{
	struct ecl_stanza *stanza = NULL;
	char *tmp;
	
	if (file->line == NULL)
		ecl_file_get_line(file);
	
	for (; file->line; ecl_file_get_line(file)) {
		for (tmp = file->line; *tmp && isspace(*tmp); tmp++);
		switch (*tmp) {
		case '#' :
			continue;
		case '[' :
			if (stanza)
				goto build_stanza_end;
			else {
				char *name;
				for (name = ++tmp; *tmp; tmp++) 
					if (*tmp == ']') {
						*tmp = '\0';
						stanza = XALLOC(ecl_stanza);
						break;
					}
					
				if (stanza == NULL) {
					ecl_error(file, "no ']'");
					continue;
				}
				stanza->name = strdup(name);
			}
			break;
		default :
			if (stanza == NULL)
				continue;
			tmp = strdup(tmp);
			xfree(file->line);
			file->line = tmp;
			ecl_file_parse_line(file, stanza);
			break;
		}
	}
	
	build_stanza_end:
	return stanza;
		
}

// add given stanza to the file->head
static void ecl_file_add_stanza(struct ecl_file *file, struct ecl_stanza *stanza)
{
	if (file->head == NULL)
		file->head = stanza;
	else {
		struct ecl_stanza *tmp = file->head;
		for (; tmp->next; SWITCH(tmp));
		tmp->next = stanza;
	}
}

void ecl_file_init(struct ecl_file *file)
{
	struct ecl_stanza *stanza;
	fseek(file->src, 0L, SEEK_SET);

	while ((stanza = ecl_file_build_stanza(file))) 
		ecl_file_add_stanza(file, stanza);
}

void ecl_show(struct ecl_file *file)
{
	struct ecl_stanza *stmp;
	struct ecl_caption *ctmp;
	printf("file:%s\nlines:%d\ncaption:\n", file->name, file->number);
	for (stmp = file->head; stmp; SWITCH(stmp)) {
		printf("[ >> %s << ]\n", stmp->name);
		for (ctmp = stmp->head; ctmp; SWITCH(ctmp)) 
			printf("'%s'='%s'\n", (char*)ctmp->name, (char *)ctmp->value);
	}
			
}

struct ecl_stanza *ecl_file_find_stanza(struct ecl_file *file, char *name)
{
	struct ecl_stanza *s = file->head;

	if (name == NULL)
		return s;

	for (; s; SWITCH(s))
		if (strcmp(name, s->name) == 0)
			break;
	return s;
}

char *ecl_stanza_find_value(struct ecl_stanza *s, char *name)
{
	struct ecl_caption *c;

	for (c = s->head; c; SWITCH(c))
		if (strcmp(name, c->name) == 0) 
			return c->value;
	return NULL;
}

void ecl_stanza_bool(struct ecl_stanza *s, char *name, int *place, int def)
{
	char *ret = ecl_stanza_find_value(s, name);
	
	if (ret == NULL) 
		*place = def;

	else if (strcmp(ret, "true") == 0 || *ret == 't' || *ret == 'y')
		*place = 1;

	else 
		*place = 0;
}

void ecl_stanza_string(struct ecl_stanza *s, char *name, char **place, char *def)
{
	char *ret = ecl_stanza_find_value(s, name);
	
	if (ret) 
		*place = strdup(ret);

	else if (def != NULL && *def)
		*place = strdup(def);	
	
	else if (def == NULL)
		*place = def;
}

void ecl_stanza_int(struct ecl_stanza *s, char *name, int *place, int def)
{
	char *ret = ecl_stanza_find_value(s, name);
	if (ret == NULL) 
		*place = def;
	else 
		*place = (int)strtol(ret, NULL, 10);
}
