/*
  otak project.
  copyright (C) 2002 
  grzegorz moskal,  <g.moskal@opengroup.org>
  license : gnu gpl version 2 			*/

#define OTAK_C_
#include "otak.h"

static int state; 
static struct tobj *rootwin;
static struct scr *screen;

int main(int argc, char **argv)
{
	if ((state = init_otak(argc, argv)) < TRUE) 
		return free_otak();

	if ((state = init_display()) < TRUE) 
		return free_otak();

	intro(state);

	do
		stream(tsys_process(NULL));
		
	while (state > QUIT);

	free_display();
	
	return free_otak();
}

void scr_show()
{
	screen->show();
}

void scr_refresh()
{	
	screen->refresh();
}

void scr_change()
{
	if (state == PEOPLE && current_man == NULL)
		return;
		
	if (state == DATA && current_man->first == NULL)
		delman_b(0, 0);	
		
	else if (state == DATA)
		mpeople_set(TLIST_ACTIVE_TITLE,
			(long)return_menuname(current_man));

	screen->hide();
	state = (state == PEOPLE) ? DATA : PEOPLE;
	screen = screen->next;
	screen->refresh();
}

void quit_otak(void)
{
	state = QUIT;
}

void wmsg(char *s)
{
	treq_info(rootwin, translate("Info message"), translate(s));
}

#define CONFIG_NAME "/etc/otak/config"
#define BYE_MSG(body, arg) \
	do {\
		errmsgf(body, arg);\
		return FORCE_QUIT;\
	} while (0)
	
static int init_otak(int argc, char **argv)
{
	long status = 0;
	
	init_rand();
	init_translate();
	init_database();

	config_name = CONFIG_NAME;

	if (parse_cmdline(argc, argv) == FALSE)
		return FORCE_QUIT;

	if (program_name == NULL)
		program_name = strglue(PACKAGE, " v ", VERSION);

	if (strcmp(config_name, CONFIG_NAME) == 0) {
		status = read_otakconf(config_name);
		config_name = strdglue(getenv("HOME"),".otak/config");
	}
	
	if ((status += read_otakconf(config_name)) == FALSE)
		BYE_MSG("Cannot read otak config '%s'", config_name);

	if (read_baseconfig(baseconfig_name) == FALSE)
		BYE_MSG("damaged base config file '%s'!", baseconfig_name);

	if (base_name != NULL && read_base(base_name) == FALSE)
		BYE_MSG("Cannot read base file '%s'!", base_name);

	if (base_name == NULL && read_basedir() == FALSE)
		return FORCE_QUIT;	

	if (try_add_email() != FALSE) 
		return QUIT;		// write database
	
	if (try_show_query() != FALSE)
		return FORCE_QUIT;	// do not write ..
		
	return TRUE;
}

static int init_tsysobjects_only(void)
{
	char *tserr = "error: recieved from tsys, %s";

	if (!winfo_create(rootwin, return_man()))
		BYE_MSG(tserr, "Cannot init win info");
	
	if (!wtext_create(rootwin))
		BYE_MSG(tserr, "Cannot init win text");

	if (!whelp_create(rootwin))
		BYE_MSG(tserr, "Cannot init win help");
		
	if (!barkeys_create(rootwin))
		BYE_MSG(tserr, "Cannot init bar keys");

	if (!scrpeople_obj_create(rootwin))
		BYE_MSG(tserr, "Cannot init scr people");

	if (!scrdata_obj_create(rootwin))
		BYE_MSG(tserr, "Cannot init scr data");

	return TRUE;
}

static int init_tsysobjects(void)
{
	if (!(rootwin = tsys_init()))
		BYE_MSG("error: %s", "can't start tsys");
		
	init_sizes();

	if (read_sizes(size_name) == FALSE)
		BYE_MSG("Cannot read windows-size file '%s'!", size_name);
	

	return init_tsysobjects_only();
}


static int init_display()
{
	current_man = first_man;
	current_stanza = first_stanza;

	screen = scrpeople_create();
	screen->next = scrdata_create();
	screen->next->next = screen;


	if (init_tsysobjects() != TRUE)
		return FORCE_QUIT;
		
	screen->refresh();
	
	return PEOPLE;
}

void free_tsysobjects_only()
{
	scrpeople_obj_free();
	scrdata_obj_free();
	barkeys_free();
	winfo_free();
	wtext_free();
	whelp_free();
}

static void free_display()
{
	wchoice_free();
	wdata_free();
	wedit_free();
	whelp_free();
	winfo_free();
	wload_free();
	wpeople_free();
	wsearch_free();
	wtext_free();
	tsys_free();
	xfree(screen->next);
	xfree(screen);
}

static void resize_display()
{
	if (t_screencols() < 30	|| t_screenrows() < 10) {
		tsys_domethod(TSYS_LOCK_SCREEN,	(long)translate("To low size"));
		return;
	}
	
	tsys_domethod(TSYS_UNLOCK_SCREEN, 0);
	free_tsysobjects_only();
	init_sizes();
	init_tsysobjects_only();

	screen->next->hide();
	screen->refresh();
}

void stream(struct tevent *e)
{
	if (e == NULL)
		return;
		
	switch (e->data) {
	case K_F1:
		whelp_show(0, 0);
		break;
	case EVENT_QUIT:
		state = FORCE_QUIT;
		sort_database = 0;
		write_database = 0;
		break;
	case K_ALT_X:
		quit_otak();
		break;
	case EVENT_TERM_RESIZE:
 		resize_display();
		tsys_domethod(TSYS_FORCE_REFRESH, 0);
 		break;
	default :
		screen->keys(e);
		break;
	}

	tevent_free(e);
}

static int free_otak(void)
{
	int ret = 2;	// FORCE_QUIT by default;
	
	if (state == QUIT) {
		write_sizes(size_name);
		if (base_name)	
			write_base(base_name);
		else 	
			write_basedir();
		ret = 0;
	} 

	free_men();
	free_stanzas();
	
	return ret;
}

