/*
  otak project.
  copyright (C) 2002 
  grzegorz moskal,  <g.moskal@opengroup.org>
  license : gnu gpl version 2 			*/

#define SCRDATA_C_
#include "scrdata.h"

int edit_mode;
struct drawer *current_drawer;
static struct tobj *bar_data;

void scrdata_hide()
{
	winfo_hide();
	wtext_hide();
	wedit_hide();
	wchoice_hide();
	wdata_hide();

	tobj_rhide(bar_data);
}

void scrdata_show()
{
	scrdata_hide();

	winfo_show();
	wdata_show();
	
	tobj_show(bar_data);
	tset(bar_data, TOBJ_ACTIVEABLE, 0);
}

void show_choice()
{
	wdata_set(TOBJ_ACTIVEABLE, 0);
	winfo_show();
	wedit_hide();
	
	wchoice_show();
}

void show_text()
{
	scrdata_show();
	wtext_show();
}

void show_edit()
{
	scrdata_show();
	wedit_show();
	wedit_settitle(current_stanza->label);
}

void scrdata_refresh()
{
	long n;
	mdata_get(TLIST_ACTIVE, &n);
	scroll_stanzas(n);
	winfo_fill(return_drawer());
	scrdata_show();
}

void editmode_service(void)
{
	int done4last = (current_stanza == last_stanza) ? 1 : 0;

	ledit_service();
	scrdata_refresh();

	if (!edit_mode) 
		return;

	if (current_stanza == last_stanza && done4last) {
		mdata_domethod(TLIST_ACTIVATE_FIRST, 0);
		edit_mode = 0;
	} else  {
		if ((current_drawer = find_drawer_by_stanza(current_stanza)))
			medit_create_m(0, 0);
		else
			show_edit();
	}
}

void edit_b(struct tobj *obj, long data)
{
	(void) obj;
	(void) data;

	edit_mode = 1;
	mdata_domethod(TLIST_ACTIVATE_FIRST, 0);
	if ((current_drawer = find_drawer_by_stanza(first_stanza)))
		ledit_fill(current_drawer->first->caption);
	editmode_service();
}

static void keys(struct tevent *e)
{
	
	
	if (tobj_doevent(bar_data, e->data))
		return;
	if (barkeys_service(e->data))
		return;
	if (wdata_service(e))
		return;
	if (wedit_service(e))
		return;
	if (wchoice_service(e))
		return;
	if (wtext_service(e))
		return;
	if (whelp_service(e))
		return;

	switch (e->data) {
	case EVENT_TTOOL_DONE:
		scrdata_refresh();
		break;
	case K_UP:
		if (edit_mode) {
			mdata_domethod(TLIST_ACTIVATE_PREV, 0);
			editmode_service();
		}
		break;
	case K_DOWN:	
		if (edit_mode) {
			mdata_domethod(TLIST_ACTIVATE_NEXT, 0);
			editmode_service();
		}
		break;
	}
}

struct scr *scrdata_create()
{
	struct scr *screen = xmalloc(sizeof(struct scr));
	screen->keys = keys;
	screen->refresh = scrdata_refresh;
	screen->hide = scrdata_hide;
	screen->show = scrdata_show;
	return screen;
}

void change_screen_b(struct tobj *obj, long data)
{
	(void)obj;
	(void)data;

	if (whelp_isshowing())
		whelp_hide();
	else
		scr_change();
}

static void new_item_m(struct tobj *obj, long data)
{
	(void)obj;
	(void)data;

	// making empty item (0) it is recognizing later
	put_item(current_drawer, 0);
	show_edit();
}



static void windata_service_b(struct tobj *obj, long data)
{
	(void)obj;
	(void)data;

	current_drawer = find_drawer_by_stanza(current_stanza);
	mchoice_flush();
	
	if (!current_drawer) 
		show_edit();
	else if (!current_drawer->pattern->multiple) {
		if (!current_drawer->pattern->command) {
			show_edit();
			ledit_fill(current_drawer->first->caption);
		} else {
			mchoice_add(translate("use it"), muse_create_m, 0);
			mchoice_add(translate("edit"),	medit_create_m, 0);
			show_choice();
		}
	} else if (!current_drawer->pattern->command) {
		mchoice_add(translate("new"), new_item_m, 0);
		mchoice_add(translate("edit"), medit_create_m, 0);
		show_choice();
	} else {
		mchoice_add(translate("use it"), muse_create_m, 0);
		mchoice_add(translate("edit"), medit_create_m, 0);
		mchoice_add(translate("new"), new_item_m, 0);
		show_choice();
	}
}

void windata_service_m(struct tobj *obj, long data)
{
	windata_service_b(obj, data);
}

#define ADD_TOOL(fnc,key,txt) add_tool(bar_data, fnc, key, translate(txt), 0)
static int bardata_create(struct tobj *rootwin)
{
	if (!(bar_data = ttoolbar_init(rootwin, toolbar_up)))
		return 0;
	if (!ADD_TOOL(change_screen_b, *back_key, "back"))
		return 0;
	if (!ADD_TOOL(edit_b, *edit_key, "edit mode"))
		return 0;
	if (!ADD_TOOL(del_item_b, *del_key, "delete"))
		return 0;
	if (!ADD_TOOL(help_b, 'h', "help"))
		return 0;
        tset(bar_data, TOBJ_HIDDEN, 1);
	
	return 1;
}

// create all objects on this screen 
int scrdata_obj_create(struct tobj *rootwin)
{
	if (!bardata_create(rootwin))
		return 0;
	if (!wchoice_create(rootwin))
		return 0;
	if (!wedit_create(rootwin))
		return 0;
	if (!wdata_create(rootwin))
		return 0;
	return 1;
}

void scrdata_obj_free(void)
{
	wchoice_free();
	wedit_free();
	wdata_free();

	tobj_free(bar_data);
	bar_data = NULL;
}
