#include "tsys.h"
#include "tobj.h"
#include "twin.h"
/* defines of events and attrs */
#include "tbutton.h"
#include <string.h>

#include "prvclass/tbutton.h"


void tbutton_dorefresh(struct tobj *obj)
{
	unsigned int len, i, x;
	struct tbutton *button;

	button = (struct tbutton *) obj->data;

	if (tobj_isactive(obj)) {
		t_textcolor(obj->fgcolor);
		t_textbackground(obj->bgcolor);
	} else {
		t_textcolor(obj->fgcolor2);
		t_textbackground(obj->bgcolor2);
	}


	len = strlen(button->label);
	x = obj->w / 2 - len / 2;
	t_putchar('[');
	for (i = 1; i < obj->w - 1; i++)
		if (i >= x && i < len + x)
			t_putchar(button->label[i - x]);
		else
			t_putchar(' ');
	t_putchar(']');
}
static int tbutton_set(struct tobj *obj, struct tobj_set *tos)
{
	struct tbutton *button;
	button = obj->data;

	switch (tos->attr) {
	case TBUTTON_LABEL:
		if (button->label)
			free(button->label);
		button->label = strdup((char *) tos->data);
		break;
	default:
		return 0;
	}
	return 1;
}

static int tbutton_get(struct tobj *obj, struct tobj_get *tog)
{
	struct tbutton *button;
	button = obj->data;

	switch (tog->attr) {
	case TBUTTON_LABEL:
		*(tog->data) = TPDATA(button->label);
		break;
	default:
		return 0;
	}
	return 1;
}
int tbutton_domethod(struct tobj *obj, long method, long data)
{
	switch (method) {
	case TOBJ_SET:
		return tbutton_set(obj, (struct tobj_set *) data);
	case TOBJ_GET:
		return tbutton_get(obj, (struct tobj_get *) data);
	default:
		return 0;
	}
}
static void tbutton_accept(struct tobj *obj)
{
	/* Change active object to next one */
	tobj_domethod(obj->parent, TWIN_ACTIVATE_NEXT, 0);

	tevent_send(obj, EVENT_TYPE_USER, EVENT_TBUTTON_PRESSED);
}

int tbutton_doevent(struct tobj *obj, int keycode)
{
	switch (keycode) {
	case ' ':
	case K_RET:
		tbutton_accept(obj);
		break;
	default:
		/* keycode not used. maybe someone will use it */
		return 0;
	}
	return 1;
}

void tbutton_cleanup(struct tobj *obj)
{
	struct tbutton *button;
	button = (struct tbutton *) obj->data;

	if (button->label)
		free(button->label);
}
void tbutton_free(struct tobj *obj)
{
	struct tbutton *button;
	button = (struct tbutton *) obj->data;

	tbutton_cleanup(obj);

	if (button)
		free(button);
}

int tbutton_setup(struct tobj *obj, struct tobj_fnc *fnc, char *label)
{
	struct tbutton *button;
	char nomem[] = "ERROR: While allocating memory for tbutton.\n";
	button = (struct tbutton *) obj->data;

	if (!(button->label = strdup(label))) {
		tsys_error(nomem);
		return 0;
	} else {
		fnc->free = &tbutton_free;
		fnc->dorefresh = &tbutton_dorefresh;
		fnc->doevent = &tbutton_doevent;
		fnc->domethod = &tbutton_domethod;

		return 1;
	}
}
struct tobj *tbutton_init(struct tobj *parent, int x, int y, int len,
			  char *label)
{
	struct tobj *obj;
	struct tbutton *button;
	char nomem[] = "ERROR: While allocating memory for tbutton.\n";

	if (!(obj = calloc(1, sizeof(struct tobj)))) {
		tsys_error(nomem);
		return 0;
	}
	if (!(button = calloc(1, sizeof(struct tbutton)))) {
		tsys_error(nomem);
		return 0;
	} else {
		obj->data = button;
		obj->x = x;
		obj->y = y;
		obj->h = 1;
		obj->w = len;

		/* setting default values */
                obj->fgcolor = TBUTTONFG;
                obj->fgcolor2 = TBUTTONFG2;
                obj->bgcolor = TBUTTONBG;
                obj->bgcolor2 =TBUTTONBG2;
		obj->activeable = 1;
		obj->parent=parent;

		if (( tbutton_setup( obj , &obj->fnc, label ) )){
			tobj_init(parent, obj);
			return obj;
		}
		else
			return 0;
	}
}
