#include <stdlib.h>
#define _DIS_H
#include "archdep.h"
#include "tdisplay.h"
#include "tsys.h"


#define T(ptr,x,y) (ptr[ (y-1)*(t_screen_width) + (x-1) ])


/* single point on virtual screen */
struct tscr_point {
	unsigned char c;

	unsigned char cols;
	/* text color + ((bg color) << 4) */
};

/* virtual screen:
 - first is a buffer
 - second is a copy (for checking where screen was changed)
*/
static struct tscr_point *tscr_buffer, *tscr_buffer2;
/* to access those tables I use define T(ptr,x,y) */


/* text color + ((bg color) << 4) */
/* Current text attrs (virtual) */
static unsigned char tscr_colors;
/* Current positions of virtual cursor */
static unsigned int tscr_x, tscr_y;
/* Dimensions of screen */
unsigned int t_screen_width, t_screen_height;


/* Position and type of hardware cursor */
static unsigned int tscr_hard_x, tscr_hard_y;
/* value of tscr_hard_type are defined in tdisplay.h */
static int tscr_hard_type;

int t_init_buffers()
{
	t_screen_width = screen_getcols();
	t_screen_height = screen_getrows();

	tscr_x = tscr_y = 1;
	tscr_hard_x = tscr_hard_y = 1;
	tscr_hard_type = TSCR_NOCURSOR;


	if (!
	    (tscr_buffer =
	     (struct tscr_point *) calloc(t_screen_width * t_screen_height,
					  sizeof(struct tscr_point))))
		return 0;
	if (!
	    (tscr_buffer2 =
	     (struct tscr_point *) calloc(t_screen_width * t_screen_height,
					  sizeof(struct tscr_point))))
		return 0;

	return 1;
}

int t_init(void)
{
	if (!screen_init())
		return 0;

	if (!t_init_buffers())
		return 0;

	return 1;
}

unsigned int t_screencols(void)
{
	return t_screen_width;
}

unsigned int t_screenrows(void)
{
	return t_screen_height;
}
void t_free_buffers()
{
	if (tscr_buffer)
		free(tscr_buffer);
	if (tscr_buffer2)
		free(tscr_buffer2);
	
}

void t_free(void)
{
	screen_free();
	t_free_buffers();
}

void t_resize()
{
	t_free_buffers();
	if (!t_init_buffers())
		exit(10);

	tmset( tsystem->rootwin,
				TOBJ_WIDTH, t_screencols(),
				TOBJ_HEIGHT, t_screenrows(), NULL );
	
	tevent_send(NULL, EVENT_TYPE_USER, EVENT_TERM_RESIZE);
	
}


void t_putchar(unsigned char c)
{
	T(tscr_buffer, tscr_x, tscr_y).c = c;
	T(tscr_buffer, tscr_x, tscr_y).cols = tscr_colors;
	if (tscr_x >= t_screen_width) {
		tscr_x = 1;
		if (tscr_y < t_screen_height)
			tscr_y += 1;
	} else
		tscr_x += 1;

}

/* Like cputs in conio.h, but with feature to dynamicly change
   colors of printed text. Using %f.ba in string, where f and b
   are numbers of colors listed below:
   0 - black
   1 - blue
   2 - green
   3 - cyan
   4 - red
   5 - magneta
   6 - brown
   7 - light gray
   8 - dark gray
   9 - light blue
   10 - light green
   11 - light cyan
   12 - light red
   13 - light magneta
   14 - yellow
   15 - white

   ex. %15.1a - white on black background
 */
static char *t_cputs_attrs(char *string)
{
	char t[10], *tmp;
	signed int fg = -1, bg = -1;
	tmp = t;

	while (*string <= '9' && *string >= '0')
		*(tmp++) = *(string++);

	if (tmp - t > 0)
		fg = atoi(t);

	if (*string == '.') {
		string++;
		tmp = t;
		while (*string <= '9' && *string >= '0')
			*(tmp++) = *(string++);

		if (tmp - t > 0)
			bg = atoi(t);
	}

	if (*string == 'a') {
		if (fg != -1)
			t_textcolor(fg);
		if (bg != -1)
			t_textbackground(bg);
	}

	return string;
}
void t_cputs(char *string)
{
	while (*string) {
		if ((*string) == '%') {
			string++;
			if ((*string) != '%')
				string = t_cputs_attrs(string);
			else
				t_putchar('%');
		} else
			t_putchar(*(string++));
	}


}

void t_lcputs(char *s, int key, unsigned char c1, unsigned char c2)
{
	t_textcolor(c1);
	while (*s) {
		if (*s == key) {
			t_textcolor(c2);
			t_putchar(*s);
			t_textcolor(c1);
			t_cputs(++s);
			break;
		} else
			t_putchar(*s);
		s++;
	}
}

void t_clrscr(void)
{
	unsigned int i, j;
	for (i = 1; i <= t_screen_width; i++) {
		for (j = 1; j <= t_screen_height; j++) {
			T(tscr_buffer, i, j).c = ' ';
			T(tscr_buffer, i, j).cols = tscr_colors;
		}
	}

}
void t_clreol(void)
{
	unsigned int i;

	for (i = tscr_x; i <= t_screen_width; i++) {
		T(tscr_buffer, i, tscr_y).c = ' ';
		T(tscr_buffer, i, tscr_y).cols = tscr_colors;
	}

}

char t_gethatch(int type)
{
	return screen_frame(5, type);
}
void t_drawframe(unsigned int width, unsigned int height, int type)
{
	unsigned int i, bx, by;
	bx = tscr_x;
	by = tscr_y;

	t_putchar(screen_frame(1, type));

	for (i = bx + 1; i < bx + width - 1; i++)
		t_putchar(screen_frame(2, type));

	t_putchar(screen_frame(3, type));


	for (i = by + 1; i < by + height - 1; i++) {
		t_gotoxy(bx, i);
		t_putchar(screen_frame(4, type));
	}

	for (i = by + 1; i < by + height - 1; i++) {
		t_gotoxy(bx + width - 1, i);
		t_putchar(screen_frame(6, type));
	}

	t_gotoxy(bx, by + height - 1);
	t_putchar(screen_frame(7, type));

	for (i = bx + 1; i < bx + width - 1; i++)
		t_putchar(screen_frame(8, type));

	t_putchar(screen_frame(9, type));
}

void t_textcolor(unsigned char col)
{
	tscr_colors = col + (tscr_colors & (0xf0));
}
void t_textbackground(unsigned char col)
{
	tscr_colors = (col << 4) + (tscr_colors & (0x0f));
}
void t_textattr(unsigned char attrs)
{
	tscr_colors = attrs;
}
void t_gotoxy(unsigned int x, unsigned int y)
{
	tscr_x = (x <= 0) ? 1 : ((x > t_screen_width) ? t_screen_width : x);
	tscr_y = (y <= 0) ? 1 : ((y > t_screen_height) ? t_screen_height : y);
}
unsigned int t_wherex(void)
{
	return tscr_x;
}
unsigned int t_wherey(void)
{
	return tscr_y;
}
static int t_checkpoint(unsigned int x, unsigned int y)
{
	if (T(tscr_buffer, x, y).c != T(tscr_buffer2, x, y).c ||
	    T(tscr_buffer, x, y).cols != T(tscr_buffer2, x, y).cols)
		return 0;
	else
		return 1;

}
static void t_syncpoint(unsigned int x, unsigned int y)
{

	T(tscr_buffer2, x, y).c = T(tscr_buffer, x, y).c;
	T(tscr_buffer2, x, y).cols = T(tscr_buffer, x, y).cols;
}
static void t_drawpoint(unsigned int x, unsigned int y)
{
	screen_gotoxy(x, y);
	screen_textattr(T(tscr_buffer, x, y).cols);
	screen_putch(T(tscr_buffer, x, y).c);
}

void t_sync(void)
{
	unsigned int i, j;

	screen_put2scr(t_screen_width, t_screen_height, tscr_buffer);
	// Syncing buffers for small updates
	for (j = 1; j <= t_screen_height; j++) {
		for (i = 1; i <= t_screen_width; i++)
			if (!t_checkpoint(i, j)) {
				t_syncpoint(i, j);
			}
	}
	screen_gotoxy(tscr_hard_x, tscr_hard_y);
	screen_setcursortype(tscr_hard_type);
}
void t_force_refresh(void)
{
	screen_force_refresh();
}
void t_sync_small(void)
{
	unsigned int i, j;
	screen_setcursortype(TSCR_NOCURSOR);

	for (j = 1; j <= t_screen_height; j++) {
		for (i = 1; i <= t_screen_width; i++)
			if (!t_checkpoint(i, j)) {
				t_drawpoint(i, j);
				t_syncpoint(i, j);
			}
	}
	screen_refresh();
	screen_gotoxy(tscr_hard_x, tscr_hard_y);
	screen_setcursortype(tscr_hard_type);
}

/* Moving and changing type of hardware cursor.
   Used for object like inputline etc  */
void t_cursormove(unsigned int x, unsigned int y)
{
	tscr_hard_x = x;
	tscr_hard_y = y;

	screen_gotoxy(tscr_hard_x, tscr_hard_y);
}
void t_cursortype(int type)
{
	tscr_hard_type = type;

	screen_setcursortype(type);
}

