#include "tsys.h"
#include "tobj.h"
#include "twin.h"
/* defines of events and methods */
#include "ttextbox.h"
#include "miscfunc.h"
#include <string.h>
#include <ctype.h>

#include "prvclass/ttextbox.h"

static int ttextbox_formattext(struct tobj *obj, char *buffer)
{
	struct ttextbox *textbox;
	unsigned long i = 0;
	char *s;

	unsigned long mem = 0, portion = 32;

	textbox = (struct ttextbox *) obj->data;

	/* freeing previous cache (if exists) */
	if (textbox->linecache) {
		free(textbox->linecache);
		textbox->linecache = NULL;
	}

	textbox->nlines = 0;
	textbox->nchars = 0;

	if (!buffer)
		return 1;

	if ((textbox->linecache = (char **) malloc(sizeof(char *) *
						   (portion + 1))))
		mem += portion;
	else
		return 0;

	textbox->linecache[textbox->nlines++] = buffer;

	s = buffer;
	/* counting lines and calculating max line lenght,
	   creating cache table
	 */
	while (*s) {
		if ((((*s) == '\n' || i >= MAXLINE) &&
		     textbox->formating == SCROLL) ||
		    (((*s) == '\n' || i >= obj->w) &&
		     (textbox->formating == CUT
		      || textbox->formating == WRAP
		      || textbox->formating == WORD))) {
			/* moving ptr back to beginning of word */
			if (textbox->formating == WORD) {
				while (!isspace(*s)
				       && s != buffer)
					s--;
			}

			/* moving ptr to end of line */
			if (textbox->formating == WRAP)
				while ((*s) != '\n' && (*s) != '\0')
					s++;

			/* setting maximal line lenght */
			if (textbox->nchars < i)
				textbox->nchars = i;

			textbox->linecache[textbox->nlines++] = s + 1;
			/* reallocing table if is going to be over-grown */
			if (textbox->nlines >= mem) {
				char **tmp;
				if ((tmp =
				     (char **) realloc(textbox->linecache,
						       sizeof(char *) *
						       (mem + portion)))) {
					textbox->linecache = tmp;
					mem += portion;
					portion *= 2;
				} else {
					free(textbox->linecache);
					return 0;
				}
			}

			i = 0;
		}
		if ((*s) == '\t')
			i += textbox->tabsize;
		else
			i++;
		s++;
	}

	return 1;
}
void ttextbox_dorefresh(struct tobj *obj)
{
	int bx, by;
	struct ttextbox *textbox;
	unsigned long i, j, k;
	int endchar = 0, endline = 0;
	char *s = NULL, *end = NULL;

	textbox = (struct ttextbox *) obj->data;

	bx = t_wherex();
	by = t_wherey();

	/* setting colour */
	t_textcolor(obj->fgcolor);
	t_textbackground(obj->bgcolor);


	for (i = 0; i < obj->h; i++) {
		/* drawing textbox box */
		t_gotoxy(bx, by + i);

		/* are we run out of lines? */
		if (i + textbox->firstline >= textbox->nlines) {
			endline = 1;
			j = 0;
		} else {
			s = textbox->linecache[i + textbox->firstline];
			if (i + textbox->firstline + 1 != textbox->nlines)
				end =
				    textbox->linecache[i +
						       textbox->firstline + 1];
			else
				end = s + strlen(s);

			/* checking if strlen < firstchar ,
			   and if there any tabs <firstchar */
			for (j = 0, k = 0; k < textbox->firstchar; j++) {
				if ((s + j) >= end) {
					endchar = 1;
				}
				if ((s + j) < end && s[j] == '\t')
					k += textbox->tabsize;
				else
					k++;
			}

			s += j;

			/* if there were tabs in unshowed part of line */
			if (k > textbox->firstchar) {
				for (j = textbox->firstchar; j < k; j++)
					t_putchar(' ');
			}
			j = k - textbox->firstchar;
		}

		for (; j < obj->w; j++) {
			if (!endline && !endchar) {
				if (*s == '\r' || *s == '\n' || *s == '\0'
				    || s == end)
					endchar = 1;
			}


			if (endline || endchar)
				t_putchar(' ');
			else if ((*s) == '\t') {
				unsigned int k;
				for (k = 0; k < textbox->tabsize; k++)
					t_putchar(' ');
				j += textbox->tabsize - 1;
			} else if (!iscntrl(*s))
				t_putchar(*s);
			else
				t_putchar('.');


			if (!endline && !endchar)
				s++;
		}
		endchar = 0;

	}
	/* turning tsystem refresh, because it's faster for
	   object, that makes many changes to it self */
	tsystem->refresh = 1;
}
static int ttextbox_set(struct tobj *obj, struct tobj_set *tos)
{
	struct ttextbox *textbox;
	textbox = obj->data;

	switch (tos->attr) {
	case TTEXTBOX_BUFFER:
		ttextbox_formattext(obj, (char *) tos->data);
		if (textbox->firstline + obj->h > textbox->nlines) {
			textbox->firstline = UNSIGNSUB(textbox->nlines, obj->h);
		}
		if (textbox->firstchar + obj->w > textbox->nchars) {
			textbox->firstchar = UNSIGNSUB(textbox->nchars, obj->w);
		}
		tobj_dorefresh(obj);
		break;
	case TTEXTBOX_FIRST_LINE:
		if (tos->data >= 0 && tos->data + obj->h <= textbox->nlines) {
			textbox->firstline = tos->data;
			tobj_dorefresh(obj);
		} else if (tos->data + obj->h > textbox->nlines) {
			if (textbox->nlines > obj->h)
				textbox->firstline = textbox->nlines - obj->h;
			else
				textbox->firstline = 0;

			tobj_dorefresh(obj);
		}
		break;
	case TTEXTBOX_FIRST_CHAR:
		if (tos->data >= 0 && tos->data + obj->w <= textbox->nchars) {
			textbox->firstchar = tos->data;
			tobj_dorefresh(obj);
		} else if (tos->data + obj->w > textbox->nchars) {
			textbox->firstchar = textbox->nchars - obj->w;
			tobj_dorefresh(obj);
		}
		break;
	case TTEXTBOX_FORMATING:{
			char *s;
			s = (textbox->linecache) ? textbox->linecache[0] : 0;
			textbox->formating = tos->data;
			ttextbox_formattext(obj, s);

			if (textbox->firstline + obj->h > textbox->nlines)
				textbox->firstline = textbox->nlines - obj->h;
			if (textbox->firstchar + obj->w > textbox->nchars)
				textbox->firstchar = textbox->nchars - obj->w;
			tobj_dorefresh(obj);
			break;
		}
	case TTEXTBOX_TABSIZE:{
			char *s;
			s = (textbox->linecache) ? textbox->linecache[0] : 0;
			textbox->tabsize = tos->data;
			ttextbox_formattext(obj, s);

			if (textbox->firstline + obj->h > textbox->nlines)
				textbox->firstline = textbox->nlines - obj->h;
			if (textbox->firstchar + obj->w > textbox->nchars)
				textbox->firstchar = textbox->nchars - obj->w;
			tobj_dorefresh(obj);
			break;
		}
	default:
		return 0;
	}

	return 1;
}

static int ttextbox_get(struct tobj *obj, struct tobj_get *tog)
{
	struct ttextbox *textbox;
	textbox = obj->data;

	switch (tog->attr) {
	case TTEXTBOX_FIRST_LINE:
		*(tog->data) = (textbox->firstline);
		return 1;
	case TTEXTBOX_FIRST_CHAR:
		*(tog->data) = (textbox->firstchar);
		return 1;
	case TTEXTBOX_LINES_MAXLEN:
		*(tog->data) = (textbox->nchars);
		return 1;
	case TTEXTBOX_LINES:
		*(tog->data) = (textbox->nlines);
		return 1;
	case TTEXTBOX_FORMATING:
		*(tog->data) = (textbox->formating);
		return 1;
	case TTEXTBOX_TABSIZE:
		*(tog->data) = (textbox->tabsize);
		return 1;
	default:
		return 0;
	}
}
int ttextbox_domethod(struct tobj *obj, long method, long data)
{
	struct ttextbox *textbox;
	textbox = (struct ttextbox *) obj->data;

	switch (method) {
	case TOBJ_SET:
		return ttextbox_set(obj, (struct tobj_set *) data);
	case TOBJ_GET:
		return ttextbox_get(obj, (struct tobj_get *) data);
	case TTEXTBOX_SCROLL_LEFT:
		if (textbox->firstchar > 0) {
			textbox->firstchar -= 1;
			tobj_dorefresh(obj);
		}
		return 1;
	case TTEXTBOX_SCROLL_RIGHT:
		if (textbox->firstchar + obj->w < (textbox->nchars)) {
			textbox->firstchar += 1;
			tobj_dorefresh(obj);
		}
		return 1;
	case TTEXTBOX_SCROLL_BEGINLINE:
		if (textbox->firstchar > 0) {
			textbox->firstchar = 0;
			tobj_dorefresh(obj);
		}
		return 1;
	case TTEXTBOX_SCROLL_ENDLINE:
		if (textbox->firstchar + obj->w < textbox->nchars) {
			textbox->firstchar = textbox->nchars - obj->w;
			tobj_dorefresh(obj);
		}
		return 1;
	case TTEXTBOX_SCROLL_UP:
		if (textbox->firstline > 0) {
			textbox->firstline -= 1;
			tobj_dorefresh(obj);
		}
		return 1;
	case TTEXTBOX_SCROLL_DOWN:
		if (textbox->firstline + obj->h < textbox->nlines) {
			textbox->firstline += 1;
			tobj_dorefresh(obj);
		}
		return 1;
	case TTEXTBOX_SCROLL_PAGEUP:
		if (textbox->firstline > obj->h) {
			textbox->firstline -= obj->h;
			tobj_dorefresh(obj);
		} else if (textbox->firstline > 0) {
			textbox->firstline = 0;
			tobj_dorefresh(obj);
		}
		return 1;
	case TTEXTBOX_SCROLL_PAGEDOWN:
		if ((textbox->firstline + obj->h) + obj->h < (textbox->nlines)) {
			textbox->firstline += obj->h;
			tobj_dorefresh(obj);
		} else if (textbox->firstline + obj->h < textbox->nlines) {
			textbox->firstline = textbox->nlines - obj->h;
			tobj_dorefresh(obj);
		}
		return 1;
	case TTEXTBOX_SCROLL_BEGIN:
		if (textbox->firstline > 0) {
			textbox->firstline = 0;
			tobj_dorefresh(obj);
		}
		return 1;
	case TTEXTBOX_SCROLL_END:
		if (textbox->firstline + obj->h < textbox->nlines) {
			textbox->firstline = textbox->nlines - obj->h;
			tobj_dorefresh(obj);
		}
		return 1;
	}
	return 0;
}


void ttextbox_cleanup(struct tobj *obj)
{
	struct ttextbox *textbox;
	textbox = (struct ttextbox *) obj->data;

	if (textbox->linecache){
		free(textbox->linecache);
		textbox->linecache= NULL;
	}
}
void ttextbox_free(struct tobj *obj)
{
	struct ttextbox *textbox;
	textbox = (struct ttextbox *) obj->data;

	ttextbox_cleanup( obj );

	if (textbox)
		free(textbox);

}

int ttextbox_setup(struct tobj *obj , struct tobj_fnc *fnc,
			   char *text, int formating)
{
	struct ttextbox *textbox;
	char nomem[] = "ERROR: While allocating memory for ttextbox.\n";
	textbox = (struct ttextbox *) obj->data;

	textbox->firstline = 0L;
	textbox->firstchar = 0L;
	textbox->formating = formating;
	textbox->tabsize = 4;

	fnc->free = &ttextbox_free;
	fnc->dorefresh = &ttextbox_dorefresh;
	fnc->domethod = &ttextbox_domethod;

	if (text && !ttextbox_formattext(obj, text)) {
		tsys_error(nomem);
		return 0;
	}

	return 1;
}
struct tobj *ttextbox_init(struct tobj *parent, int x, int y, int w, int h,
			   char *text, int formating)
{
	struct tobj *obj;
	struct ttextbox *textbox;
	char nomem[] = "ERROR: While allocating memory for ttextbox.\n";

	if (!(obj = calloc(1, sizeof(struct tobj)))) {
		tsys_error(nomem);
		return 0;
	}
	if (!(textbox = calloc(1, sizeof(struct ttextbox)))) {
		tsys_error(nomem);
		return 0;
	} else {
		obj->data = textbox;

		obj->x = x;
		obj->y = y;
		obj->h = h;
		obj->w = w;

		/* setting default values */
                obj->fgcolor = TTEXTBOXFG;
                obj->fgcolor2 = TTEXTBOXFG2;
                obj->bgcolor = TTEXTBOXBG;
                obj->bgcolor2 =TTEXTBOXBG2;

		obj->activeable = 1;
		obj->parent = parent;

		if (( ttextbox_setup( obj, &obj->fnc , text, formating ) )){
			tobj_init(parent, obj);
			return obj;
		} else
			return 0;
	}
}
