/*
  otak project.
  copyright (C) 2002 
  grzegorz moskal,  <g.moskal@opengroup.org>
  license : gnu gpl version 2 			*/

#define WLOAD_C_
#include "wload.h"

static int state, show;
static struct tobj  *line_load, *cmenu_load, *win_load;

int wload_service(struct tevent *e)
{
	if (e->sender != line_load && e->sender != cmenu_load 
		&& e->sender != win_load)
		return 0;
		
	switch (e->data) {
	case EVENT_TLIST_REJECT:
	case EVENT_TMENU_REJECT:
	case EVENT_TINPUT_REJECT:
		scrpeople_show();
		return 1;
	case EVENT_TINPUT_ACCEPT:
		lload_service();
		return 1;
	}
	return 0;

}


static int singlemsg()
{
	if (base_name) {
		wmsg(" Otak`s worikng in single mode [-b option were given]");
		return 1;
	}
	return 0;
}
void newbase_b(struct tobj *obj, long data)
{
	(void) obj;
	(void) data;

	if (singlemsg())
		return;
	lload_flush();
	wload_settitle("entry name");
	tobj_domethod(win_load, TWIN_ACTIVATE_CHILD, (long)line_load);
	wload_show();
}

void loadbase_b(struct tobj *obj, long data)
{
	(void) obj;
	(void) data;

	if (singlemsg())
		return;
	state = LOAD;
	wload_settitle("load database");
	tobj_domethod(win_load, TWIN_ACTIVATE_CHILD, (long)cmenu_load);
	wload_show();
}

void moveman_b(struct tobj *obj, long data)
{
	(void) obj;
	(void) data;

	if (singlemsg() || !first_base || 
		!first_base->next || !current_man)
		return;
	state = MOVE;
	wload_settitle("where move this field ?");
	tobj_domethod(win_load, TWIN_ACTIVATE_CHILD, (long)cmenu_load);
	wload_show();
}

void delbase_b(struct tobj *obj, long data)
{
	(void) obj;
	(void) data;

	if (singlemsg() || !first_base)
		return;
	state = DELETE;
	wload_settitle("delete database");
	tobj_domethod(win_load, TWIN_ACTIVATE_CHILD, (long)cmenu_load);
	wload_show();
}

void lload_service()
{
	char * name = (char *)txget(line_load, TINPUT_BUFFER_CURRENT);
	char *tmp;
	if (name == NULL || *name == '\0')
		return;
	set_basepointers();
	clear_manpointers();  
	add_base(name);
	last_base->data = NEW;
	tmp = last_base->name;
	last_base->name = strdglue(basedir_name, name);
	xfree(tmp);
	add_menuobj(cmenu_load, name, mload_service_m, 0);
	tobj_domethod(cmenu_load, TLIST_ACTIVATE_LAST, 0);
	current_base = last_base;
	set_basepointers();
	loadbase();
}

static struct base *scroll_bases(void)
{
	struct base *b = first_base;
	long n;
	tget(cmenu_load, TLIST_ACTIVE, &n);
	for (; n; SWITCH(b), n--);
	return b;
}

static void loadbase()
{
	char *title;
	set_basepointers();
	mpeople_free();
	current_base = scroll_bases();
	set_manpointers();	
	if (!mpeople_create())
		errexit("Cannot reinit menu people");
	title = (current_base) ?
		current_base->name+strlen(basedir_name)+1 : "no database!";
	wpeople_set(TWIN_TITLE, (long)translate(title));
	scrpeople_refresh();
}

static void mload_entrydel(void)
{
	long n;
	tget(cmenu_load, TLIST_ACTIVE, &n);
	tobj_domethod(cmenu_load, TLIST_ENTRY_DEL, n);
	tobj_domethod(cmenu_load, TLIST_ACTIVATE_LAST, 0);
}

static void moveman(struct base *src, struct base *dest)
{
	struct man *tmp = current_man;
	if (src == dest)
		return;
		
	delman_b(0, MOVE);	// delete man from mpeople;
	set_basepointers();
	tmp->next = 0;
	current_base = dest;
	set_manpointers();
	add_man(tmp);
	set_basepointers();

	current_base = src;
	set_manpointers();
}

static void deletebase()
{
	struct base *b;
	set_basepointers();
	b = scroll_bases();
	if (b != current_base)  {
		del_base(b);
		mload_entrydel();
		scrpeople_refresh();	
	} else {
		del_base(b);
		mload_entrydel();
		current_base = 0;
		loadbase();
	}
		
}

static void mload_service_m(struct tobj *obj, long data)
{
	(void) obj;
	(void) data;

	switch (state) {
	case LOAD:
		loadbase();
		break;
	case DELETE:
		deletebase();
		break;
	case MOVE:
		moveman(current_base, scroll_bases());
		scrpeople_refresh();
		break;
	}

}

int wload_create(struct tobj *rootwin)
{
	DEFINE_SIZES(Wload);
	struct tcmenu_entry mtab[] = {{ TMENU_END , NULL , 0 }};
	struct base *j;
	int i = strlen(basedir_name) + 1;

	if (!(win_load = twin_init(rootwin, x, y, w, h, translate("databases"))))
		return 0;
		
	if (!(cmenu_load = tcmenu_init(win_load, 1, 1, w-2, h-2, mtab)))
		return 0;

	for (j = first_base; j; j = j->next) 
		if (add_menuobj(cmenu_load, j->name+i, mload_service_m, 0) == 0)
			return 0;

	if (!lload_flush())
		return 0;
		
	return 1;
}

static void wload_settitle(char *name)
{
	tset(win_load, TWIN_TITLE, (long)translate(name));
}

void wload_show()
{
	tobj_show(win_load);
	show = 1;
}

void wload_hide()
{
	tobj_rhide(win_load);
	show = 0;
}

int wload_isshowing()
{
	return show;
}

static int lload_flush()
{
	int reinit = (line_load) ? 1 : 0;
	if (reinit)
		tobj_free(line_load);
	if (!(line_load = tinput_init(win_load, 1,
		get_h(Wload)-2, get_w(Wload)-2, "", 30))) {
		if (reinit)
			errexit("Cannot reinit line load.\n");
		return 0;
	}
	return 1;
}

void wload_free()
{
	set_size(Wload, win_load);
	tobj_free(win_load);
	win_load = NULL;
	cmenu_load = NULL;
	line_load = NULL; 
}
